@@ -16,7 +16,6 @@
 #include <config.h>
 #include <stdbool.h>
 
-#include "lflow.h"
 #include "local_data.h"
 #include "lport.h"
 #include "mac-cache.h"
@@ -24,7 +23,6 @@
 #include "openvswitch/vlog.h"
 #include "ovn/logical-fields.h"
 #include "ovn-sb-idl.h"
-#include "pinctrl.h"
 
 VLOG_DEFINE_THIS_MODULE(mac_cache);
 
@@ -38,12 +36,6 @@ mac_binding_data_hash(const struct mac_binding_data *mb_data);
 static inline bool
 mac_binding_data_equals(const struct mac_binding_data *a,
                         const struct mac_binding_data *b);
-static void mac_binding_update_log(const char *action,
-                                   const struct mac_binding_data *,
-                                   bool print_times,
-                                   const struct mac_cache_threshold *,
-                                   int64_t idle_age_ms,
-                                   uint64_t since_updated_ms);
 static uint32_t
 fdb_data_hash(const struct fdb_data *fdb_data);
 static inline bool
@@ -91,21 +83,7 @@ mac_cache_threshold_add(struct mac_cache_data *data,
     threshold = xmalloc(sizeof *threshold);
     threshold->dp_key = dp->tunnel_key;
     threshold->value = value;
-    threshold->dump_period = (3 * value) / 16;
-    threshold->cooldown_period = (3 * value) / 16;
-
-    /* (cooldown_period + dump_period) is the maximum time the timestamp may
-     * be not updated for an entry with IP + MAC combination from which we see
-     * incoming traffic.  For the entry that is used only in Tx direction
-     * (e.g., an entry for a default gateway of the chassis) this time is
-     * doubled, because an ARP/ND probe will need to be sent first and the
-     * (cooldown_period + dump_period) will be the maximum time between such
-     * probes.  Hence, 2 * (cooldown_period + dump_period) should be less than
-     * a threshold, otherwise we may fail to update an active MAC binding in
-     * time and risk it being removed.  Giving it an extra 1/10 of the time
-     * for all the processing that needs to happen. */
-    ovs_assert(2 * (threshold->cooldown_period + threshold->dump_period)
-               < (9 * value) / 10);
+    threshold->dump_period = (3 * value) / 4;
 
     hmap_insert(&data->thresholds, &threshold->hmap_node, dp->tunnel_key);
 }
@@ -164,26 +142,25 @@ mac_cache_thresholds_clear(struct mac_cache_data *data)
 }
 
 /* MAC binding. */
-void
+struct mac_binding *
 mac_binding_add(struct hmap *map, struct mac_binding_data mb_data,
-                const struct sbrec_mac_binding *smb, long long timestamp)
-{
+                long long timestamp) {
+
     struct mac_binding *mb = mac_binding_find(map, &mb_data);
     if (!mb) {
         mb = xmalloc(sizeof *mb);
+        mb->sbrec_mb = NULL;
         hmap_insert(map, &mb->hmap_node, mac_binding_data_hash(&mb_data));
     }
 
     mb->data = mb_data;
-    mb->sbrec = smb;
     mb->timestamp = timestamp;
-    mac_binding_update_log("Added", &mb_data, false, NULL, 0, 0);
+
+    return mb;
 }
 
 void
-mac_binding_remove(struct hmap *map, struct mac_binding *mb)
-{
-    mac_binding_update_log("Removed", &mb->data, false, NULL, 0, 0);
+mac_binding_remove(struct hmap *map, struct mac_binding *mb) {
     hmap_remove(map, &mb->hmap_node);
     free(mb);
 }
@@ -203,38 +180,25 @@ mac_binding_find(const struct hmap *map,
     return NULL;
 }
 
-bool
-mac_binding_data_parse(struct mac_binding_data *data,
-                       uint32_t dp_key, uint32_t port_key,
-                       const char *ip_str, const char *mac_str)
-{
-    struct eth_addr mac;
-    struct in6_addr ip;
-
-    if (!ip46_parse(ip_str, &ip) || !eth_addr_from_string(mac_str, &mac)) {
-        static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(5, 1);
-        VLOG_WARN_RL(&rl, "Couldn't parse MAC binding: ip=%s, mac=%s",
-                     ip_str, mac_str);
-        return false;
-    }
-
-    mac_binding_data_init(data, dp_key, port_key, ip, mac);
-    return true;
-}
-
 bool
 mac_binding_data_from_sbrec(struct mac_binding_data *data,
-                            const struct sbrec_mac_binding *mb)
+                            const struct sbrec_mac_binding *mb,
+                            struct ovsdb_idl_index *sbrec_pb_by_name)
 {
-    /* This explicitly sets the port_key to 0 as port_binding tunnel_keys
-     * can change.  Instead use add the SB.MAC_Binding UUID as key; this
-     * makes the mac_binding_data key unique. */
-    if (!mac_binding_data_parse(data, mb->datapath->tunnel_key, 0,
-                                mb->ip, mb->mac)) {
+    const struct sbrec_port_binding *pb =
+            lport_lookup_by_name(sbrec_pb_by_name, mb->logical_port);
+
+    if (!pb || !pb->datapath || !ip46_parse(mb->ip, &data->ip) ||
+        !eth_addr_from_string(mb->mac, &data->mac)) {
+        static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(5, 1);
+        VLOG_WARN_RL(&rl, "Couldn't parse MAC binding: ip=%s, mac=%s, "
+                     "logical_port=%s", mb->ip, mb->mac, mb->logical_port);
         return false;
     }
 
-    data->cookie = mb->header_.uuid.parts[0];
+    data->dp_key = mb->datapath->tunnel_key;
+    data->port_key = pb->tunnel_key;
+
     return true;
 }
 
@@ -247,33 +211,6 @@ mac_bindings_clear(struct hmap *map)
     }
 }
 
-static void
-mac_binding_data_to_string(const struct mac_binding_data *data,
-                           struct ds *out_data)
-{
-    char ip[INET6_ADDRSTRLEN];
-
-    if (!ipv6_string_mapped(ip, &data->ip)) {
-        return;
-    }
-    ds_put_format(out_data, "cookie: 0x%08"PRIx64", "
-                            "datapath-key: %"PRIu32", "
-                            "port-key: %"PRIu32", "
-                            "ip: %s, mac: " ETH_ADDR_FMT,
-                  data->cookie, data->dp_key, data->port_key,
-                  ip, ETH_ADDR_ARGS(data->mac));
-}
-
-void
-mac_bindings_to_string(const struct hmap *map, struct ds *out_data)
-{
-    struct mac_binding *mb;
-    HMAP_FOR_EACH (mb, hmap_node, map) {
-        mac_binding_data_to_string(&mb->data, out_data);
-        ds_put_char(out_data, '\n');
-    }
-}
-
 bool
 sb_mac_binding_updated(const struct sbrec_mac_binding *mb)
 {
@@ -308,7 +245,8 @@ mac_binding_lookup(struct ovsdb_idl_index *sbrec_mac_binding_by_lport_ip,
 
 /* FDB. */
 struct fdb *
-fdb_add(struct hmap *map, struct fdb_data fdb_data) {
+fdb_add(struct hmap *map, struct fdb_data fdb_data, long long timestamp)
+{
     struct fdb *fdb = fdb_find(map, &fdb_data);
 
     if (!fdb) {
@@ -318,6 +256,7 @@ fdb_add(struct hmap *map, struct fdb_data fdb_data) {
     }
 
     fdb->data = fdb_data;
+    fdb->timestamp = timestamp;
 
     return fdb;
 }
@@ -406,15 +345,12 @@ mac_binding_stats_process_flow_stats(struct ovs_list *stats_list,
 
     stats->idle_age_ms = ofp_stats->idle_age * 1000;
     stats->data.mb = (struct mac_binding_data) {
-        .cookie = ntohll(ofp_stats->cookie),
-        /* The port_key must be zero to match mac_binding_data_from_sbrec. */
-        .port_key = 0,
+        .port_key = ofp_stats->match.flow.regs[MFF_LOG_INPORT - MFF_REG0],
         .dp_key = ntohll(ofp_stats->match.flow.metadata),
         .mac = ofp_stats->match.flow.dl_src
     };
 
-    if (ofp_stats->match.flow.dl_type == htons(ETH_TYPE_IP) ||
-        ofp_stats->match.flow.dl_type == htons(ETH_TYPE_ARP)) {
+    if (ofp_stats->match.flow.dl_type == htons(ETH_TYPE_IP)) {
         stats->data.mb.ip = in6_addr_mapped_ipv4(ofp_stats->match.flow.nw_src);
     } else {
         stats->data.mb.ip = ofp_stats->match.flow.ipv6_src;
@@ -423,38 +359,9 @@ mac_binding_stats_process_flow_stats(struct ovs_list *stats_list,
     ovs_list_push_back(stats_list, &stats->list_node);
 }
 
-static void
-mac_binding_update_log(const char *action,
-                       const struct mac_binding_data *data,
-                       bool print_times,
-                       const struct mac_cache_threshold *threshold,
-                       int64_t idle_age_ms, uint64_t since_updated_ms)
-{
-    if (!VLOG_IS_DBG_ENABLED()) {
-        return;
-    }
-
-    struct ds s = DS_EMPTY_INITIALIZER;
-
-    ds_put_cstr(&s, action);
-    ds_put_cstr(&s, " MAC binding (");
-    mac_binding_data_to_string(data, &s);
-    if (print_times) {
-        ds_put_format(&s, "), last update: %"PRIu64"ms ago,"
-                          " idle age: %"PRIi64"ms, threshold: %"PRIu64"ms",
-                      since_updated_ms, idle_age_ms, threshold->value);
-    } else {
-        ds_put_char(&s, ')');
-    }
-    VLOG_DBG("%s.", ds_cstr_ro(&s));
-    ds_destroy(&s);
-}
-
 void
-mac_binding_stats_run(
-        struct rconn *swconn OVS_UNUSED,
-        struct ovsdb_idl_index *sbrec_port_binding_by_name OVS_UNUSED,
-        struct ovs_list *stats_list, uint64_t *req_delay, void *data)
+mac_binding_stats_run(struct ovs_list *stats_list, uint64_t *req_delay,
+                      void *data)
 {
     struct mac_cache_data *cache_data = data;
     long long timewall_now = time_wall_msec();
@@ -464,43 +371,26 @@ mac_binding_stats_run(
         struct mac_binding *mb = mac_binding_find(&cache_data->mac_bindings,
                                                   &stats->data.mb);
         if (!mb) {
-            mac_binding_update_log("Not found in the cache:", &stats->data.mb,
-                                   false, NULL, 0, 0);
             free(stats);
             continue;
         }
 
-        uint64_t since_updated_ms = timewall_now - mb->sbrec->timestamp;
         struct mac_cache_threshold *threshold =
                 mac_cache_threshold_find(cache_data, mb->data.dp_key);
 
         /* If "idle_age" is under threshold it means that the mac binding is
-         * used on this chassis. */
-        if (stats->idle_age_ms < threshold->value) {
-            if (since_updated_ms >= threshold->cooldown_period) {
-                mac_binding_update_log("Updating active", &mb->data, true,
-                                       threshold, stats->idle_age_ms,
-                                       since_updated_ms);
-                sbrec_mac_binding_set_timestamp(mb->sbrec, timewall_now);
-            } else {
-                /* Postponing the update to avoid sending database transactions
-                 * too frequently. */
-                mac_binding_update_log("Not updating active", &mb->data, true,
-                                       threshold, stats->idle_age_ms,
-                                       since_updated_ms);
-            }
-        } else {
-            mac_binding_update_log("Not updating non-active", &mb->data, true,
-                                   threshold, stats->idle_age_ms,
-                                   since_updated_ms);
+         * used on this chassis. Also make sure that we don't update the
+         * timestamp more than once during the dump period. */
+        if (stats->idle_age_ms < threshold->value &&
+                (timewall_now - mb->sbrec_mb->timestamp) >=
+            threshold->dump_period) {
+            sbrec_mac_binding_set_timestamp(mb->sbrec_mb, timewall_now);
         }
+
         free(stats);
     }
 
     mac_cache_update_req_delay(&cache_data->thresholds, req_delay);
-    if (*req_delay) {
-        VLOG_DBG("MAC binding statistics dalay: %"PRIu64, *req_delay);
-    }
 }
 
 /* FDB stat processing. */
@@ -520,39 +410,9 @@ fdb_stats_process_flow_stats(struct ovs_list *stats_list,
     ovs_list_push_back(stats_list, &stats->list_node);
 }
 
-static void
-fdb_update_log(const char *action,
-               const struct fdb_data *data,
-               bool print_times,
-               const struct mac_cache_threshold *threshold,
-               int64_t idle_age_ms, uint64_t since_updated_ms)
-{
-    if (!VLOG_IS_DBG_ENABLED()) {
-        return;
-    }
-
-    struct ds s = DS_EMPTY_INITIALIZER;
-
-    ds_put_cstr(&s, action);
-    ds_put_format(&s, " FDB entry (datapath-key: %"PRIu32", "
-                      "port-key: %"PRIu32", mac: " ETH_ADDR_FMT,
-                  data->dp_key, data->port_key, ETH_ADDR_ARGS(data->mac));
-    if (print_times) {
-        ds_put_format(&s, "), last update: %"PRIu64"ms ago,"
-                          " idle age: %"PRIi64"ms, threshold: %"PRIu64"ms",
-                      since_updated_ms, idle_age_ms, threshold->value);
-    } else {
-        ds_put_char(&s, ')');
-    }
-    VLOG_DBG("%s.", ds_cstr_ro(&s));
-    ds_destroy(&s);
-}
-
 void
-fdb_stats_run(struct rconn *swconn OVS_UNUSED,
-              struct ovsdb_idl_index *sbrec_port_binding_by_name OVS_UNUSED,
-              struct ovs_list *stats_list,
-              uint64_t *req_delay, void *data)
+fdb_stats_run(struct ovs_list *stats_list, uint64_t *req_delay,
+              void *data)
 {
     struct mac_cache_data *cache_data = data;
     long long timewall_now = time_wall_msec();
@@ -562,43 +422,26 @@ fdb_stats_run(struct rconn *swconn OVS_UNUSED,
         struct fdb *fdb = fdb_find(&cache_data->fdbs, &stats->data.fdb);
 
         if (!fdb) {
-            fdb_update_log("Not found in the cache:", &stats->data.fdb,
-                           false, NULL, 0, 0);
             free(stats);
             continue;
         }
 
-        uint64_t since_updated_ms = timewall_now - fdb->sbrec_fdb->timestamp;
         struct mac_cache_threshold *threshold =
                 mac_cache_threshold_find(cache_data, fdb->data.dp_key);
 
-        /* If "idle_age" is under threshold it means that the fdb entry is
-         * used on this chassis. */
-        if (stats->idle_age_ms < threshold->value) {
-            if (since_updated_ms >= threshold->cooldown_period) {
-                fdb_update_log("Updating active", &fdb->data, true,
-                               threshold, stats->idle_age_ms,
-                               since_updated_ms);
-                sbrec_fdb_set_timestamp(fdb->sbrec_fdb, timewall_now);
-            } else {
-                /* Postponing the update to avoid sending database transactions
-                 * too frequently. */
-                fdb_update_log("Not updating active", &fdb->data, true,
-                               threshold, stats->idle_age_ms,
-                               since_updated_ms);
-            }
-        } else {
-            fdb_update_log("Not updating non-active", &fdb->data, true,
-                           threshold, stats->idle_age_ms, since_updated_ms);
+        /* If "idle_age" is under threshold it means that the mac binding is
+         * used on this chassis. Also make sure that we don't update the
+         * timestamp more than once during the dump period. */
+        if (stats->idle_age_ms < threshold->value &&
+                (timewall_now - fdb->sbrec_fdb->timestamp) >=
+            threshold->dump_period) {
+            sbrec_fdb_set_timestamp(fdb->sbrec_fdb, timewall_now);
         }
 
         free(stats);
     }
 
     mac_cache_update_req_delay(&cache_data->thresholds, req_delay);
-    if (*req_delay) {
-        VLOG_DBG("FDB entry statistics dalay: %"PRIu64, *req_delay);
-    }
 }
 
 /* Packet buffering. */
@@ -765,7 +608,7 @@ mac_cache_stats_destroy(struct ovs_list *stats_list)
 static uint32_t
 mac_binding_data_hash(const struct mac_binding_data *mb_data)
 {
-    uint32_t hash = hash_uint64(mb_data->cookie);
+    uint32_t hash = 0;
 
     hash = hash_add(hash, mb_data->port_key);
     hash = hash_add(hash, mb_data->dp_key);
@@ -778,10 +621,9 @@ static inline bool
 mac_binding_data_equals(const struct mac_binding_data *a,
                         const struct mac_binding_data *b)
 {
-    return a->cookie == b->cookie &&
-           a->port_key == b->port_key &&
+    return a->port_key == b->port_key &&
            a->dp_key == b->dp_key &&
-           ipv6_addr_equals(&a->ip, &b->ip);
+            ipv6_addr_equals(&a->ip, &b->ip);
 }
 
 static uint32_t
@@ -903,114 +745,3 @@ buffered_packets_db_lookup(struct buffered_packets *bp, struct ds *ip,
 
     eth_addr_from_string(smb->mac, mac);
 }
-
-void
-mac_binding_probe_stats_process_flow_stats(
-        struct ovs_list *stats_list,
-        struct ofputil_flow_stats *ofp_stats)
-{
-    struct mac_cache_stats *stats = xmalloc(sizeof *stats);
-
-    stats->idle_age_ms = ofp_stats->idle_age * 1000;
-    stats->data.mb = (struct mac_binding_data) {
-        .cookie = ntohll(ofp_stats->cookie),
-        /* The port_key must be zero to match mac_binding_data_from_sbrec. */
-        .port_key = 0,
-        .dp_key = ntohll(ofp_stats->match.flow.metadata),
-    };
-
-    if (ofp_stats->match.flow.regs[0]) {
-        stats->data.mb.ip =
-            in6_addr_mapped_ipv4(htonl(ofp_stats->match.flow.regs[0]));
-    } else {
-        ovs_be128 ip6 = hton128(flow_get_xxreg(&ofp_stats->match.flow, 1));
-        memcpy(&stats->data.mb.ip, &ip6, sizeof stats->data.mb.ip);
-    }
-
-    ovs_list_push_back(stats_list, &stats->list_node);
-}
-
-void
-mac_binding_probe_stats_run(
-        struct rconn *swconn,
-        struct ovsdb_idl_index *sbrec_port_binding_by_name,
-        struct ovs_list *stats_list,
-        uint64_t *req_delay, void *data)
-{
-    long long timewall_now = time_wall_msec();
-    struct mac_cache_data *cache_data = data;
-
-    struct mac_cache_stats *stats;
-    LIST_FOR_EACH_POP (stats, list_node, stats_list) {
-        struct mac_binding *mb = mac_binding_find(&cache_data->mac_bindings,
-                                                  &stats->data.mb);
-        if (!mb) {
-            mac_binding_update_log("Probe: not found in the cache:",
-                                   &stats->data.mb, false, NULL, 0, 0);
-            free(stats);
-            continue;
-        }
-
-        struct mac_cache_threshold *threshold =
-                mac_cache_threshold_find(cache_data, mb->data.dp_key);
-        uint64_t since_updated_ms = timewall_now - mb->sbrec->timestamp;
-        const struct sbrec_mac_binding *sbrec = mb->sbrec;
-
-        if (stats->idle_age_ms > threshold->value) {
-            mac_binding_update_log("Not sending ARP/ND request for non-active",
-                                   &mb->data, true, threshold,
-                                   stats->idle_age_ms, since_updated_ms);
-            free(stats);
-            continue;
-        }
-
-        if (since_updated_ms < threshold->cooldown_period) {
-            mac_binding_update_log(
-                    "Not sending ARP/ND request for recently updated",
-                    &mb->data, true, threshold, stats->idle_age_ms,
-                    since_updated_ms);
-            free(stats);
-            continue;
-        }
-
-        const struct sbrec_port_binding *pb =
-            lport_lookup_by_name(sbrec_port_binding_by_name,
-                                 sbrec->logical_port);
-        if (!pb) {
-            free(stats);
-            continue;
-        }
-
-        struct lport_addresses laddr;
-        if (!extract_lsp_addresses(pb->mac[0], &laddr)) {
-            free(stats);
-            continue;
-        }
-
-        bool is_mb_v4 = IN6_IS_ADDR_V4MAPPED(&mb->data.ip);
-        if ((is_mb_v4 && laddr.n_ipv4_addrs)
-                || (!is_mb_v4 && laddr.n_ipv6_addrs)) {
-            struct in6_addr local =
-                is_mb_v4 ? in6_addr_mapped_ipv4(laddr.ipv4_addrs[0].addr)
-                         : laddr.ipv6_addrs[0].addr;
-
-            mac_binding_update_log("Sending ARP/ND request for active",
-                                   &mb->data, true, threshold,
-                                   stats->idle_age_ms, since_updated_ms);
-
-            send_self_originated_neigh_packet(swconn,
-                                              sbrec->datapath->tunnel_key,
-                                              pb->tunnel_key, laddr.ea,
-                                              &local, &mb->data.ip,
-                                              OFTABLE_LOCAL_OUTPUT);
-        }
-
-        free(stats);
-        destroy_lport_addresses(&laddr);
-    }
-
-    mac_cache_update_req_delay(&cache_data->thresholds, req_delay);
-    if (*req_delay) {
-        VLOG_DBG("MAC probe binding statistics delay: %"PRIu64, *req_delay);
-    }
-}
