@@ -33,14 +33,12 @@
 #include "seq.h"
 #include "socket-util.h"
 #include "statctrl.h"
-#include "stopwatch.h"
 
 VLOG_DEFINE_THIS_MODULE(statctrl);
 
 enum stat_type {
     STATS_MAC_BINDING = 0,
     STATS_FDB,
-    STATS_MAC_BINDING_PROBE,
     STATS_MAX,
 };
 
@@ -64,38 +62,27 @@ struct stats_node {
                                struct ofputil_flow_stats *ofp_stats);
     /* Function to process the parsed stats.
      * This function runs in main thread locked behind mutex. */
-    void (*run)(struct rconn *swconn,
-                struct ovsdb_idl_index *sbrec_port_binding_by_name,
-                struct ovs_list *stats_list,
-                uint64_t *req_delay, void *data);
-    /* Name of the stats node corresponding stopwatch. */
-    const char *stopwatch_name;
+    void (*run)(struct ovs_list *stats_list, uint64_t *req_delay, void *data);
 };
 
 #define STATS_NODE(NAME, REQUEST, DESTROY, PROCESS, RUN)                   \
-    do {                                                                   \
-        statctrl_ctx.nodes[STATS_##NAME] = (struct stats_node) {           \
-            .request = REQUEST,                                            \
-            .xid = 0,                                                      \
-            .next_request_timestamp = INT64_MAX,                           \
-            .request_delay = 0,                                            \
-            .stats_list =                                                  \
-                OVS_LIST_INITIALIZER(                                      \
-                    &statctrl_ctx.nodes[STATS_##NAME].stats_list),         \
-            .destroy = DESTROY,                                            \
-            .process_flow_stats = PROCESS,                                 \
-            .run = RUN,                                                    \
-            .stopwatch_name = OVS_STRINGIZE(stats_##NAME),                 \
-        };                                                                 \
-        stopwatch_create(OVS_STRINGIZE(stats_##NAME), SW_MS);              \
-    } while (0)
+    statctrl_ctx.nodes[STATS_##NAME] = (struct stats_node) {               \
+        .request = REQUEST,                                                \
+        .xid = 0,                                                          \
+        .next_request_timestamp = INT64_MAX,                               \
+        .request_delay = 0,                                                \
+        .stats_list =                                                      \
+            OVS_LIST_INITIALIZER(                                          \
+                &statctrl_ctx.nodes[STATS_##NAME].stats_list),             \
+        .destroy = DESTROY,                                                \
+        .process_flow_stats = PROCESS,                                     \
+        .run = RUN                                                         \
+    };
 
 struct statctrl_ctx {
     /* OpenFlow connection to the switch. */
     struct rconn *swconn;
 
-    char *br_int;
-
     pthread_t thread;
     struct latch exit_latch;
 
@@ -109,8 +96,6 @@ static struct statctrl_ctx statctrl_ctx;
 static struct ovs_mutex mutex;
 
 static void *statctrl_thread_handler(void *arg);
-static void statctrl_rconn_setup(struct rconn *swconn, char *conn_target)
-    OVS_REQUIRES(mutex);
 static void statctrl_handle_rconn_msg(struct rconn *swconn,
                                       struct statctrl_ctx *ctx,
                                       struct ofpbuf *msg);
@@ -123,8 +108,6 @@ static void statctrl_send_request(struct rconn *swconn,
                                   struct statctrl_ctx *ctx)
     OVS_REQUIRES(mutex);
 static void statctrl_notify_main_thread(struct statctrl_ctx *ctx);
-static void statctrl_set_conn_target(const char *br_int_name)
-    OVS_REQUIRES(mutex);
 static void statctrl_wait_next_request(struct statctrl_ctx *ctx)
     OVS_REQUIRES(mutex);
 static bool statctrl_update_next_request_timestamp(struct stats_node *node,
@@ -135,9 +118,7 @@ static bool statctrl_update_next_request_timestamp(struct stats_node *node,
 void
 statctrl_init(void)
 {
-    /* OpenFlow connection to the switch. */
     statctrl_ctx.swconn = rconn_create(0, 0, DSCP_DEFAULT, 1 << OFP15_VERSION);
-    statctrl_ctx.br_int = NULL;
     latch_init(&statctrl_ctx.exit_latch);
     ovs_mutex_init(&mutex);
     statctrl_ctx.thread_seq = seq_create();
@@ -164,18 +145,6 @@ statctrl_init(void)
     STATS_NODE(FDB, fdb_request, mac_cache_stats_destroy,
                fdb_stats_process_flow_stats, fdb_stats_run);
 
-    struct ofputil_flow_stats_request mac_binding_probe_request = {
-            .cookie = htonll(0),
-            .cookie_mask = htonll(0),
-            .out_port = OFPP_ANY,
-            .out_group = OFPG_ANY,
-            .table_id = OFTABLE_MAC_BINDING,
-    };
-    STATS_NODE(MAC_BINDING_PROBE, mac_binding_probe_request,
-               mac_cache_stats_destroy,
-               mac_binding_probe_stats_process_flow_stats,
-               mac_binding_probe_stats_run);
-
     statctrl_ctx.thread = ovs_thread_create("ovn_statctrl",
                                             statctrl_thread_handler,
                                             &statctrl_ctx);
@@ -183,18 +152,13 @@ statctrl_init(void)
 
 void
 statctrl_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
-             struct ovsdb_idl_index *sbrec_port_binding_by_name,
              struct mac_cache_data *mac_cache_data)
 {
     if (!ovnsb_idl_txn) {
         return;
     }
 
-    void *node_data[STATS_MAX] = {
-        mac_cache_data,
-        mac_cache_data,
-        mac_cache_data
-    };
+    void *node_data[STATS_MAX] = {mac_cache_data, mac_cache_data};
 
     bool schedule_updated = false;
     long long now = time_msec();
@@ -204,11 +168,7 @@ statctrl_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
         struct stats_node *node = &statctrl_ctx.nodes[i];
         uint64_t prev_delay = node->request_delay;
 
-        stopwatch_start(node->stopwatch_name, time_msec());
-        node->run(statctrl_ctx.swconn,
-                  sbrec_port_binding_by_name, &node->stats_list,
-                  &node->request_delay, node_data[i]);
-        stopwatch_stop(node->stopwatch_name, time_msec());
+        node->run(&node->stats_list, &node->request_delay, node_data[i]);
 
         schedule_updated |=
                 statctrl_update_next_request_timestamp(node, now, prev_delay);
@@ -221,11 +181,14 @@ statctrl_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
 }
 
 void
-statctrl_update(const char *br_int_name)
+statctrl_update_swconn(const char *target, int probe_interval)
 {
-    ovs_mutex_lock(&mutex);
-    statctrl_set_conn_target(br_int_name);
-    ovs_mutex_unlock(&mutex);
+    if (ovn_update_swconn_at(statctrl_ctx.swconn, target,
+                             probe_interval, "statctrl")) {
+        /* Notify statctrl thread that integration bridge
+         * target is set/changed. */
+        seq_change(statctrl_ctx.thread_seq);
+    }
 }
 
 void
@@ -254,7 +217,6 @@ statctrl_destroy(void)
     pthread_join(statctrl_ctx.thread, NULL);
     latch_destroy(&statctrl_ctx.exit_latch);
     rconn_destroy(statctrl_ctx.swconn);
-    free(statctrl_ctx.br_int);
     seq_destroy(statctrl_ctx.thread_seq);
     seq_destroy(statctrl_ctx.main_seq);
 
@@ -273,10 +235,6 @@ statctrl_thread_handler(void *arg)
     struct rconn *swconn = ctx->swconn;
 
     while (!latch_is_set(&ctx->exit_latch)) {
-        ovs_mutex_lock(&mutex);
-        statctrl_rconn_setup(swconn, ctx->br_int);
-        ovs_mutex_unlock(&mutex);
-
         rconn_run(swconn);
         uint64_t new_seq = seq_read(ctx->thread_seq);
 
@@ -312,25 +270,6 @@ statctrl_thread_handler(void *arg)
     return NULL;
 }
 
-static void
-statctrl_rconn_setup(struct rconn *swconn, char *br_int)
-    OVS_REQUIRES(mutex)
-{
-    if (!br_int) {
-        rconn_disconnect(swconn);
-        return;
-    }
-
-    char *conn_target = xasprintf("unix:%s/%s.mgmt", ovs_rundir(), br_int);
-
-    if (strcmp(conn_target, rconn_get_target(swconn))) {
-        VLOG_INFO("%s: connecting to switch", conn_target);
-        rconn_connect(swconn, conn_target, conn_target);
-    }
-
-    free(conn_target);
-}
-
 static void
 statctrl_handle_rconn_msg(struct rconn *swconn, struct statctrl_ctx *ctx,
                           struct ofpbuf *msg)
@@ -435,23 +374,6 @@ statctrl_notify_main_thread(struct statctrl_ctx *ctx)
     }
 }
 
-static void
-statctrl_set_conn_target(const char *br_int_name)
-    OVS_REQUIRES(mutex)
-{
-    if (!br_int_name) {
-        return;
-    }
-
-
-    if (!statctrl_ctx.br_int || strcmp(statctrl_ctx.br_int, br_int_name)) {
-        free(statctrl_ctx.br_int);
-        statctrl_ctx.br_int = xstrdup(br_int_name);
-        /* Notify statctrl thread that integration bridge is set/changed. */
-        seq_change(statctrl_ctx.thread_seq);
-    }
-}
-
 static void
 statctrl_wait_next_request(struct statctrl_ctx *ctx)
     OVS_REQUIRES(mutex)
