@@ -137,7 +137,7 @@ az_run(struct ic_context *ctx)
      * "ovn-ic-sbctl destroy avail <az>". */
     static char *az_name;
     const struct icsbrec_availability_zone *az;
-    if (ctx->ovnisb_txn && az_name && strcmp(az_name, nb_global->name)) {
+    if (az_name && strcmp(az_name, nb_global->name)) {
         ICSBREC_AVAILABILITY_ZONE_FOR_EACH (az, ctx->ovnisb_idl) {
             /* AZ name update locally need to update az in ISB. */
             if (nb_global->name[0] && !strcmp(az->name, az_name)) {
@@ -1029,24 +1029,27 @@ prefix_is_link_local(struct in6_addr *prefix, unsigned int plen)
 }
 
 static bool
-prefix_is_black_listed(const struct smap *nb_options,
-                       struct in6_addr *prefix,
-                       unsigned int plen)
+prefix_is_deny_listed(const struct smap *nb_options,
+                      struct in6_addr *prefix,
+                      unsigned int plen)
 {
-    const char *blacklist = smap_get(nb_options, "ic-route-blacklist");
-    if (!blacklist || !blacklist[0]) {
-        return false;
+    const char *denylist = smap_get(nb_options, "ic-route-denylist");
+    if (!denylist || !denylist[0]) {
+        denylist = smap_get(nb_options, "ic-route-blacklist");
+        if (!denylist || !denylist[0]) {
+            return false;
+        }
     }
     struct in6_addr bl_prefix;
     unsigned int bl_plen;
     char *cur, *next, *start;
-    next = start = xstrdup(blacklist);
+    next = start = xstrdup(denylist);
     bool matched = false;
     while ((cur = strsep(&next, ",")) && *cur) {
         if (!ip46_parse_cidr(cur, &bl_prefix, &bl_plen)) {
             static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(5, 1);
             VLOG_WARN_RL(&rl, "Bad format in nb_global options:"
-                         "ic-route-blacklist: %s. CIDR expected.", cur);
+                         "ic-route-denylist: %s. CIDR expected.", cur);
             continue;
         }
 
@@ -1068,10 +1071,13 @@ prefix_is_black_listed(const struct smap *nb_options,
                 continue;
             }
         } else {
-            struct in6_addr bl_mask = ipv6_create_mask(bl_plen);
-            struct in6_addr m_prefix = ipv6_addr_bitand(prefix, &bl_mask);
-            struct in6_addr m_bl_prefix = ipv6_addr_bitand(&bl_prefix,
-                                                           &bl_mask);
+            struct in6_addr mask = ipv6_create_mask(plen);
+            /* First calculate the difference between bl_prefix and prefix, so
+             * use the bl mask to ensure prefixes are correctly validated.
+             * e.g.: 2005:1734:5678::/50 is a subnet of 2005:1234::/21 */
+            struct in6_addr m_prefixes = ipv6_addr_bitand(prefix, &bl_prefix);
+            struct in6_addr m_prefix = ipv6_addr_bitand(&m_prefixes, &mask);
+            struct in6_addr m_bl_prefix = ipv6_addr_bitand(&bl_prefix, &mask);
             if (!ipv6_addr_equals(&m_prefix, &m_bl_prefix)) {
                 continue;
             }
@@ -1106,7 +1112,7 @@ route_need_advertise(const char *policy,
         return false;
     }
 
-    if (prefix_is_black_listed(nb_options, prefix, plen)) {
+    if (prefix_is_deny_listed(nb_options, prefix, plen)) {
         return false;
     }
     return true;
@@ -1120,8 +1126,6 @@ add_to_routes_ad(struct hmap *routes_ad, const struct in6_addr prefix,
                  const struct nbrec_logical_router_static_route *nb_route,
                  const struct nbrec_logical_router *nb_lr)
 {
-    ovs_assert(nb_route || nb_lrp);
-
     if (route_table == NULL) {
         route_table = "";
     }
@@ -1142,15 +1146,9 @@ add_to_routes_ad(struct hmap *routes_ad, const struct in6_addr prefix,
         hmap_insert(routes_ad, &ic_route->node, hash);
     } else {
         static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(5, 1);
-        const char *msg_fmt = "Duplicate %s route advertisement was "
-                              "suppressed! NB %s uuid: "UUID_FMT;
-        if (nb_route) {
-            VLOG_WARN_RL(&rl, msg_fmt, origin, "route",
-                         UUID_ARGS(&nb_route->header_.uuid));
-        } else {
-            VLOG_WARN_RL(&rl, msg_fmt, origin, "lrp",
-                         UUID_ARGS(&nb_lrp->header_.uuid));
-        }
+        VLOG_WARN_RL(&rl, "Duplicate route advertisement was suppressed! NB "
+                     "route uuid: "UUID_FMT,
+                     UUID_ARGS(&nb_route->header_.uuid));
     }
 }
 
@@ -1267,52 +1265,11 @@ route_has_local_gw(const struct nbrec_logical_router *lr,
     return false;
 }
 
-static bool
-lrp_has_neighbor_in_ts(const struct nbrec_logical_router_port *lrp,
-                       struct in6_addr *nexthop)
-{
-    if (!lrp || !nexthop) {
-        return false;
-    }
-
-    struct lport_addresses lrp_networks;
-    if (!extract_lrp_networks(lrp, &lrp_networks)) {
-        destroy_lport_addresses(&lrp_networks);
-        return false;
-    }
-
-    if (IN6_IS_ADDR_V4MAPPED(nexthop)) {
-        ovs_be32 neigh_prefix_v4 = in6_addr_get_mapped_ipv4(nexthop);
-        for (size_t i = 0; i < lrp_networks.n_ipv4_addrs; i++) {
-            struct ipv4_netaddr address = lrp_networks.ipv4_addrs[i];
-            if (address.network == (neigh_prefix_v4 & address.mask)) {
-                destroy_lport_addresses(&lrp_networks);
-                return true;
-            }
-        }
-    } else {
-        for (size_t i = 0; i < lrp_networks.n_ipv6_addrs; i++) {
-            struct ipv6_netaddr address = lrp_networks.ipv6_addrs[i];
-            struct in6_addr neigh_prefix = ipv6_addr_bitand(nexthop,
-                                                            &address.mask);
-            if (ipv6_addr_equals(&address.network, &neigh_prefix)) {
-                destroy_lport_addresses(&lrp_networks);
-                return true;
-            }
-        }
-    }
-
-    destroy_lport_addresses(&lrp_networks);
-    return false;
-}
-
 static bool
 route_need_learn(const struct nbrec_logical_router *lr,
                  const struct icsbrec_route *isb_route,
                  struct in6_addr *prefix, unsigned int plen,
-                 const struct smap *nb_options,
-                 const struct nbrec_logical_router_port *ts_lrp,
-                 struct in6_addr *nexthop)
+                 const struct smap *nb_options)
 {
     if (!smap_get_bool(nb_options, "ic-route-learn", false)) {
         return false;
@@ -1327,7 +1284,7 @@ route_need_learn(const struct nbrec_logical_router *lr,
         return false;
     }
 
-    if (prefix_is_black_listed(nb_options, prefix, plen)) {
+    if (prefix_is_deny_listed(nb_options, prefix, plen)) {
         return false;
     }
 
@@ -1337,10 +1294,6 @@ route_need_learn(const struct nbrec_logical_router *lr,
         return false;
     }
 
-    if (!lrp_has_neighbor_in_ts(ts_lrp, nexthop)) {
-        return false;
-    }
-
     return true;
 }
 
@@ -1362,20 +1315,6 @@ get_lrp_name_by_ts_port_name(struct ic_context *ctx, const char *ts_port_name)
     return smap_get(&nb_lsp->options, "router-port");
 }
 
-static const struct nbrec_logical_router_port *
-get_lrp_by_lrp_name(struct ic_context *ctx, const char *lrp_name)
-{
-    const struct nbrec_logical_router_port *lrp;
-    const struct nbrec_logical_router_port *lrp_key =
-        nbrec_logical_router_port_index_init_row(ctx->nbrec_lrp_by_name);
-    nbrec_logical_router_port_index_set_name(lrp_key, lrp_name);
-    lrp = nbrec_logical_router_port_index_find(ctx->nbrec_lrp_by_name,
-                                               lrp_key);
-    nbrec_logical_router_port_index_destroy_row(lrp_key);
-
-    return lrp;
-}
-
 static const char *
 get_route_table_by_lrp_name(struct ic_context *ctx, const char *lrp_name)
 {
@@ -1393,67 +1332,6 @@ get_route_table_by_lrp_name(struct ic_context *ctx, const char *lrp_name)
     return "";  /* <main> route table */
 }
 
-static const struct nbrec_logical_router_port *
-find_lrp_of_nexthop(struct ic_context *ctx,
-                    const struct icsbrec_route *isb_route)
-{
-    const struct nbrec_logical_router_port *lrp;
-    const struct nbrec_logical_switch *ls;
-    ls = find_ts_in_nb(ctx, isb_route->transit_switch);
-    if (!ls) {
-        return NULL;
-    }
-
-    struct in6_addr nexthop;
-    if (!ip46_parse(isb_route->nexthop, &nexthop)) {
-        return NULL;
-    }
-
-    for (size_t i = 0; i < ls->n_ports; i++) {
-        char *lsp_name = ls->ports[i]->name;
-        const char *lrp_name = get_lrp_name_by_ts_port_name(ctx,
-                                                            lsp_name);
-        if (!lrp_name) {
-            continue;
-        }
-
-        lrp = get_lrp_by_lrp_name(ctx, lrp_name);
-        if (!lrp) {
-            continue;
-        }
-
-        struct lport_addresses lrp_networks;
-        if (!extract_lrp_networks(lrp, &lrp_networks)) {
-            destroy_lport_addresses(&lrp_networks);
-            continue;
-        }
-
-        if (IN6_IS_ADDR_V4MAPPED(&nexthop)) {
-            ovs_be32 nexthop_v4 = in6_addr_get_mapped_ipv4(&nexthop);
-            for (size_t i_v4 = 0; i_v4  < lrp_networks.n_ipv4_addrs; i_v4++) {
-                struct ipv4_netaddr address = lrp_networks.ipv4_addrs[i_v4];
-                if (address.addr == nexthop_v4) {
-                    destroy_lport_addresses(&lrp_networks);
-                    return lrp;
-                }
-            }
-        } else {
-            for (size_t i_v6 = 0; i_v6 < lrp_networks.n_ipv6_addrs; i_v6++) {
-                struct ipv6_netaddr address = lrp_networks.ipv6_addrs[i_v6];
-                struct in6_addr nexthop_v6 = ipv6_addr_bitand(&nexthop,
-                                                              &address.mask);
-                if (ipv6_addr_equals(&address.network, &nexthop_v6)) {
-                    destroy_lport_addresses(&lrp_networks);
-                    return lrp;
-                }
-            }
-        }
-        destroy_lport_addresses(&lrp_networks);
-    }
-
-    return NULL;
-}
-
 static bool
 lrp_is_ts_port(struct ic_context *ctx, struct ic_router_info *ic_lr,
                const char *lrp_name)
@@ -1483,11 +1361,9 @@ sync_learned_routes(struct ic_context *ctx,
 
     const char *lrp_name, *ts_route_table;
     const struct icsbrec_port_binding *isb_pb;
-    const struct nbrec_logical_router_port *lrp;
     for (int i = 0; i < ic_lr->n_isb_pbs; i++) {
         isb_pb = ic_lr->isb_pbs[i];
         lrp_name = get_lrp_name_by_ts_port_name(ctx, isb_pb->logical_port);
-        lrp = get_lrp_by_lrp_name(ctx, lrp_name);
         ts_route_table = get_route_table_by_lrp_name(ctx, lrp_name);
 
         isb_route_key = icsbrec_route_index_init_row(ctx->icsbrec_route_by_ts);
@@ -1530,7 +1406,7 @@ sync_learned_routes(struct ic_context *ctx,
                 continue;
             }
             if (!route_need_learn(ic_lr->lr, isb_route, &prefix, plen,
-                                  &nb_global->options, lrp, &nexthop)) {
+                                  &nb_global->options)) {
                 continue;
             }
 
@@ -1821,7 +1697,7 @@ delete_orphan_ic_routes(struct ic_context *ctx,
             ctx->icnbrec_transit_switch_by_name, t_sw_key);
         icnbrec_transit_switch_index_destroy_row(t_sw_key);
 
-        if (!t_sw || !find_lrp_of_nexthop(ctx, isb_route)) {
+        if (!t_sw) {
             static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(5, 1);
             VLOG_INFO_RL(&rl, "Deleting orphan ICDB:Route: %s->%s (%s, rtb:%s,"
                          " transit switch: %s)", isb_route->ip_prefix,
@@ -2244,8 +2120,6 @@ main(int argc, char *argv[])
                          &nbrec_logical_router_col_external_ids);
 
     ovsdb_idl_add_table(ovnnb_idl_loop.idl, &nbrec_table_logical_router_port);
-    ovsdb_idl_add_column(ovnnb_idl_loop.idl,
-                         &nbrec_logical_router_port_col_mac);
     ovsdb_idl_add_column(ovnnb_idl_loop.idl,
                          &nbrec_logical_router_port_col_name);
     ovsdb_idl_add_column(ovnnb_idl_loop.idl,
