@@ -23,6 +23,7 @@
 #include "expr.h"
 #include "openvswitch/dynamic-string.h"
 #include "openvswitch/hmap.h"
+#include "openvswitch/ofp-actions.h"
 #include "openvswitch/uuid.h"
 #include "util.h"
 #include "ovn/features.h"
@@ -65,7 +66,7 @@ struct collector_set_ids;
     OVNACT(EXCHANGE,          ovnact_move)            \
     OVNACT(DEC_TTL,           ovnact_null)            \
     OVNACT(CT_NEXT,           ovnact_ct_next)         \
-    OVNACT(CT_COMMIT_V1,      ovnact_ct_commit_v1)    \
+    /* CT_COMMIT_V1 is not supported anymore. */      \
     OVNACT(CT_COMMIT_V2,      ovnact_nest)            \
     OVNACT(CT_COMMIT_TO_ZONE, ovnact_ct_commit_to_zone) \
     OVNACT(CT_DNAT,           ovnact_ct_nat)          \
@@ -96,6 +97,8 @@ struct collector_set_ids;
     OVNACT(LOOKUP_ND_IP,      ovnact_lookup_mac_bind_ip) \
     OVNACT(PUT_DHCPV4_OPTS,   ovnact_put_opts)        \
     OVNACT(PUT_DHCPV6_OPTS,   ovnact_put_opts)        \
+    OVNACT(DHCPV4_RELAY_REQ_CHK,  ovnact_dhcp_relay)      \
+    OVNACT(DHCPV4_RELAY_RESP_CHK, ovnact_dhcp_relay)      \
     OVNACT(SET_QUEUE,         ovnact_set_queue)       \
     OVNACT(DNS_LOOKUP,        ovnact_result)          \
     OVNACT(LOG,               ovnact_log)             \
@@ -128,10 +131,6 @@ struct collector_set_ids;
     OVNACT(CHK_LB_AFF,        ovnact_result)          \
     OVNACT(SAMPLE,            ovnact_sample)          \
     OVNACT(MAC_CACHE_USE,     ovnact_null)            \
-    OVNACT(CT_ORIG_NW_DST,    ovnact_result)          \
-    OVNACT(CT_ORIG_IP6_DST,   ovnact_result)          \
-    OVNACT(CT_ORIG_TP_DST,    ovnact_result)          \
-    OVNACT(CT_STATE_SAVE,     ovnact_result)
 
 /* enum ovnact_type, with a member OVNACT_<ENUM> for each action. */
 enum OVS_PACKED_ENUM ovnact_type {
@@ -261,17 +260,9 @@ struct ovnact_push_pop {
 /* OVNACT_CT_NEXT. */
 struct ovnact_ct_next {
     struct ovnact ovnact;
-    bool dnat_zone;
     uint8_t ltable;                /* Logical table ID of next table. */
 };
 
-/* OVNACT_CT_COMMIT_V1. */
-struct ovnact_ct_commit_v1 {
-    struct ovnact ovnact;
-    uint32_t ct_mark, ct_mark_mask;
-    ovs_be128 ct_label, ct_label_mask;
-};
-
 /* Type of NAT used for the particular action.
  * UNSPEC translates to applying NAT that works for both directions. */
 enum ovnact_ct_nat_type {
@@ -402,6 +393,14 @@ struct ovnact_put_opts {
     size_t n_options;
 };
 
+/* OVNACT_DHCP_RELAY. */
+struct ovnact_dhcp_relay {
+    struct ovnact ovnact;
+    struct expr_field dst;      /* 1-bit destination field. */
+    ovs_be32 relay_ipv4;
+    ovs_be32 server_ipv4;
+};
+
 /* Valid arguments to SET_QUEUE action.
  *
  * QDISC_MIN_QUEUE_ID is the default queue, so user-defined queues should
@@ -415,11 +414,10 @@ struct ovnact_set_queue {
     uint16_t queue_id;
 };
 
-/* OVNACT_DNS_LOOKUP, OVNACT_CHK_LB_HAIRPIN, OVNACT_CHK_LB_HAIRPIN_REPLY,
- * OVNACT_CT_ORIG_NW_DST, CT_ORIG_IP6_DST, CT_ORIG_TP_DST */
+/* OVNACT_DNS_LOOKUP, OVNACT_CHK_LB_HAIRPIN, OVNACT_CHK_LB_HAIRPIN_REPLY. */
 struct ovnact_result {
     struct ovnact ovnact;
-    struct expr_field dst;      /* destination field. */
+    struct expr_field dst;      /* 1-bit destination field. */
 };
 
 /* OVNACT_LOG. */
@@ -500,13 +498,15 @@ struct ovnact_lookup_fdb {
 /* OVNACT_SAMPLE */
 struct ovnact_sample {
     struct ovnact ovnact;
-    uint16_t probability;       /* probability over UINT16_MAX. */
-    uint8_t obs_domain_id;      /* most significant byte of the
-                                   observation domain id. The other 24 bits
-                                   will come from the datapath's tunnel key. */
-    uint32_t collector_set_id;  /* colector_set_id. */
-    uint32_t obs_point_id;      /* observation point id. */
-    bool use_cookie;            /* use cookie as obs_point_id */
+    uint16_t probability;               /* probability over UINT16_MAX. */
+    uint8_t obs_domain_id;              /* most significant byte of the
+                                           observation domain id. The other
+                                           24 bits will come from the
+                                           datapath's tunnel key. */
+    uint32_t collector_set_id;          /* colector_set_id. */
+    struct expr_field obs_point_id_src; /* observation point id source reg */
+    uint32_t obs_point_id;              /* observation point id */
+    bool use_cookie;                    /* use cookie as obs_point_id */
 };
 
 /* OVNACT_COMMIT_ECMP_NH. */
@@ -530,6 +530,19 @@ struct ovnact_commit_lb_aff {
     uint16_t timeout;
 };
 
+#define OVN_FIELD_NOTE_MAGIC "ovn"
+
+struct ovn_field_note_header {
+    OFPACT_PADDED_MEMBERS(
+        char magic[4];
+        ovs_be16 type;   /* The type of ovn field note, based
+                          * on 'enum ovn_field_id'. */
+    );
+    uint8_t data[];
+};
+
+BUILD_ASSERT_DECL(sizeof(struct ovn_field_note_header) == 8);
+
 /* Internal use by the helpers below. */
 void ovnact_init(struct ovnact *, enum ovnact_type, size_t len);
 void *ovnact_put(struct ofpbuf *, enum ovnact_type, size_t len);
@@ -588,21 +601,13 @@ void *ovnact_put(struct ofpbuf *, enum ovnact_type, size_t len);
 OVNACTS
 #undef OVNACT
 
-/* List of OVN logical action opcodes.
- *
- * This macro is used directly only internally by this header and actions.c,
- * but the list is still of interest to developers.
- *
- * Each ACTION_OPCODE invocation has the <ENUM> parameter, used below in the
- * enum definition of ACTION_OPCODE_<ENUM>, and elsewhere.
- */
-#define ACTION_OPCODES                                                        \
+enum action_opcode {
     /* "arp { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ARP)                                                        \
-                                                                              \
+     */
+    ACTION_OPCODE_ARP,
+
     /* "put_arp(port, ip, mac)"
      *
      * Arguments are passed through the packet metadata and data, as follows:
@@ -610,9 +615,9 @@ OVNACTS
      *     MFF_REG0 = ip
      *     MFF_LOG_INPORT = port
      *     MFF_ETH_SRC = mac
-     */                                                                       \
-    ACTION_OPCODE(PUT_ARP)                                                    \
-                                                                              \
+     */
+    ACTION_OPCODE_PUT_ARP,
+
     /* "result = put_dhcp_opts(offer_ip, option, ...)".
      *
      * Arguments follow the action_header, in this format:
@@ -620,15 +625,15 @@ OVNACTS
      *   - A 32-bit integer specifying a bit offset within the result field.
      *   - The 32-bit DHCP offer IP.
      *   - Any number of DHCP options.
-     */                                                                       \
-    ACTION_OPCODE(PUT_DHCP_OPTS)                                              \
-                                                                              \
+     */
+    ACTION_OPCODE_PUT_DHCP_OPTS,
+
     /* "nd_na { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ND_NA)                                                      \
-                                                                              \
+     */
+    ACTION_OPCODE_ND_NA,
+
     /* "put_nd(port, ip6, mac)"
      *
      * Arguments are passed through the packet metadata and data, as follows:
@@ -636,149 +641,160 @@ OVNACTS
      *     MFF_XXREG0 = ip6
      *     MFF_LOG_INPORT = port
      *     MFF_ETH_SRC = mac
-     */                                                                       \
-    ACTION_OPCODE(PUT_ND)                                                     \
-                                                                              \
+     */
+    ACTION_OPCODE_PUT_ND,
+
     /* "result = put_dhcpv6_opts(option, ...)".
      *
      * Arguments follow the action_header, in this format:
      *   - A 32-bit or 64-bit OXM header designating the result field.
      *   - A 32-bit integer specifying a bit offset within the result field.
      *   - Any number of DHCPv6 options.
-     */                                                                       \
-    ACTION_OPCODE(PUT_DHCPV6_OPTS)                                            \
-                                                                              \
+     */
+    ACTION_OPCODE_PUT_DHCPV6_OPTS,
+
     /* "result = dns_lookup()".
      * Arguments follow the action_header, in this format:
      *   - A 32-bit or 64-bit OXM header designating the result field.
      *   - A 32-bit integer specifying a bit offset within the result field.
      *
-     */                                                                       \
-    ACTION_OPCODE(DNS_LOOKUP)                                                 \
-                                                                              \
+     */
+    ACTION_OPCODE_DNS_LOOKUP,
+
     /* "log(arguments)".
      *
      * Arguments are as follows:
      *   - An 8-bit verdict.
      *   - An 8-bit severity.
      *   - A variable length string containing the name.
-     */                                                                       \
-    ACTION_OPCODE(LOG)                                                        \
-                                                                              \
+     */
+    ACTION_OPCODE_LOG,
+
     /* "result = put_nd_ra_opts(option, ...)".
      * Arguments follow the action_header, in this format:
      *   - A 32-bit or 64-bit OXM header designating the result field.
      *   - A 32-bit integer specifying a bit offset within the result field.
      *   - Any number of ICMPv6 options.
-     */                                                                       \
-    ACTION_OPCODE(PUT_ND_RA_OPTS)                                             \
-                                                                              \
+     */
+    ACTION_OPCODE_PUT_ND_RA_OPTS,
+
     /* "nd_ns { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ND_NS)                                                      \
-                                                                              \
+     */
+    ACTION_OPCODE_ND_NS,
+
     /* "icmp4 { ...actions... } and icmp6 { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ICMP)                                                       \
-                                                                              \
+     */
+    ACTION_OPCODE_ICMP,
+
     /* "tcp_reset { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(TCP_RESET)                                                  \
-                                                                              \
+     */
+    ACTION_OPCODE_TCP_RESET,
+
     /* "nd_na_router { ...actions... }" with rso flag 'ND_RSO_ROUTER' set.
-     *
-     * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ND_NA_ROUTER)                                               \
-                                                                              \
-    /* MTU value (to put in the icmp4 header field - frag_mtu) follow the
-     * action header. */                                                      \
-    ACTION_OPCODE(PUT_ICMP4_FRAG_MTU)                                         \
-                                                                              \
+        *
+        * The actions, in OpenFlow 1.3 format, follow the action_header.
+        */
+    ACTION_OPCODE_ND_NA_ROUTER,
+
+     /* MTU value (to put in the icmp4 header field - frag_mtu) follow the
+     * action header. */
+    ACTION_OPCODE_PUT_ICMP4_FRAG_MTU,
+
     /* "icmp4_error { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ICMP4_ERROR)                                                \
-                                                                              \
-    /* "trigger_event (event_type)" */                                        \
-    ACTION_OPCODE(EVENT)                                                      \
+     */
+    ACTION_OPCODE_ICMP4_ERROR,
+
+    /* "trigger_event (event_type)" */
+    ACTION_OPCODE_EVENT,
+
     /* "igmp".
      *
      * Snoop IGMP, learn the multicast participants
-     */                                                                       \
-    ACTION_OPCODE(IGMP)                                                       \
-                                                                              \
+     */
+    ACTION_OPCODE_IGMP,
+
     /* "bind_vport(vport, vport_parent)".
      *
      *   'vport' follows the action_header, in the format - 32-bit field.
      *   'vport_parent' is passed through the packet metadata as
      *    MFF_LOG_INPORT.
-     */                                                                       \
-    ACTION_OPCODE(BIND_VPORT)                                                 \
-                                                                              \
+     */
+    ACTION_OPCODE_BIND_VPORT,
+
     /* "handle_svc_check(port)"."
      *
      * Arguments are passed through the packet metadata and data, as follows:
      *
      *     MFF_LOG_INPORT = port
-     */                                                                       \
-    ACTION_OPCODE(HANDLE_SVC_CHECK)                                           \
-                                                                              \
+     */
+    ACTION_OPCODE_HANDLE_SVC_CHECK,
     /* handle_dhcpv6_reply { ...actions ...}."
      *
      *  The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(DHCP6_SERVER)                                               \
-                                                                              \
+     */
+    ACTION_OPCODE_DHCP6_SERVER,
+
     /* "icmp6_error { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(ICMP6_ERROR)                                                \
-                                                                              \
+     */
+    ACTION_OPCODE_ICMP6_ERROR,
+
     /* MTU value (to put in the icmp6 header field - frag_mtu) follow the
-     * action header. */                                                      \
-    ACTION_OPCODE(PUT_ICMP6_FRAG_MTU)                                         \
-                                                                              \
+     * action header. */
+    ACTION_OPCODE_PUT_ICMP6_FRAG_MTU,
+
     /* "reject { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(REJECT)                                                     \
-                                                                              \
+     */
+    ACTION_OPCODE_REJECT,
+
     /* handle_bfd_msg { ...actions ...}."
      *
      *  The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(BFD_MSG)                                                    \
-                                                                              \
+     */
+    ACTION_OPCODE_BFD_MSG,
+
     /* "sctp_abort { ...actions... }".
      *
      * The actions, in OpenFlow 1.3 format, follow the action_header.
-     */                                                                       \
-    ACTION_OPCODE(SCTP_ABORT)                                                 \
-                                                                              \
-    /* put_fdb(inport, eth.src). */                                           \
-    ACTION_OPCODE(PUT_FDB)                                                    \
-                                                                              \
-    /* activation_strategy_rarp() */                                          \
-    ACTION_OPCODE(ACTIVATION_STRATEGY_RARP)                                   \
-                                                                              \
-    /* multicast group split buffer action. */                                \
-    ACTION_OPCODE(MG_SPLIT_BUF)
+     */
+    ACTION_OPCODE_SCTP_ABORT,
 
+    /* put_fdb(inport, eth.src).
+     */
+    ACTION_OPCODE_PUT_FDB,
 
-enum action_opcode {
-#define ACTION_OPCODE(ENUM) ACTION_OPCODE_##ENUM,
-    ACTION_OPCODES
-#undef ACTION_OPCODE
+    /* activation_strategy_rarp() */
+    ACTION_OPCODE_ACTIVATION_STRATEGY_RARP,
+
+    /* multicast group split buffer action. */
+    ACTION_OPCODE_MG_SPLIT_BUF,
+
+    /* "dhcp_relay_req_chk(relay_ip, server_ip)".
+     *
+     * Arguments follow the action_header, in this format:
+     *   - The 32-bit DHCP relay IP.
+     *   - The 32-bit DHCP server IP.
+     */
+    ACTION_OPCODE_DHCP_RELAY_REQ_CHK,
+
+    /* "dhcp_relay_resp_chk(relay_ip, server_ip)".
+     *
+     * Arguments follow the action_header, in this format:
+     *   - The 32-bit DHCP relay IP.
+     *   - The 32-bit DHCP server IP.
+     */
+    ACTION_OPCODE_DHCP_RELAY_RESP_CHK,
 };
 
 /* Header. */
@@ -917,14 +933,6 @@ struct ovnact_encode_params {
                                     this determines which CT zone to use */
     uint32_t mac_cache_use_table; /* OpenFlow table for 'mac_cache_use'
                                    * to resubmit. */
-    uint32_t ct_nw_dst_load_table; /* OpenFlow table for 'ct_nw_dst'
-                                    *  to resubmit. */
-    uint32_t ct_ip6_dst_load_table; /* OpenFlow table for 'ct_ip6_dst'
-                                     *  to resubmit. */
-    uint32_t ct_tp_dst_load_table; /* OpenFlow table for 'ct_tp_dst'
-                                    *  to resubmit. */
-    uint32_t ct_state_save_table; /* OpenFlow table for saving ct_state to
-                                   * register. */
 };
 
 void ovnacts_encode(const struct ovnact[], size_t ovnacts_len,
@@ -941,4 +949,7 @@ void encode_finish_controller_op(size_t ofs, struct ofpbuf *ofpacts);
 void encode_controller_op(enum action_opcode opcode, uint32_t meter_id,
                           struct ofpbuf *ofpacts);
 
+size_t encode_start_ovn_field_note(enum ovn_field_id, struct ofpbuf *ofpacts);
+void encode_finish_ovn_field_note(size_t offset, struct ofpbuf *ofpacts);
+
 #endif /* ovn/actions.h */
