@@ -29,8 +29,7 @@
 #define OVN_FEATURE_LS_DPG_COLUMN "ls-dpg-column"
 #define OVN_FEATURE_CT_COMMIT_NAT_V2 "ct-commit-nat-v2"
 #define OVN_FEATURE_CT_COMMIT_TO_ZONE "ct-commit-to-zone"
-#define OVN_FEATURE_CT_NEXT_ZONE "ct-next-zone"
-#define OVN_FEATURE_CT_STATE_SAVE "ct-state-save"
+#define OVN_FEATURE_SAMPLE_WITH_REGISTERS "ovn-sample-with-registers"
 
 /* OVS datapath supported features.  Based on availability OVN might generate
  * different types of openflows.
@@ -40,6 +39,8 @@ enum ovs_feature_support_bits {
     OVS_DP_METER_SUPPORT_BIT,
     OVS_CT_TUPLE_FLUSH_BIT,
     OVS_DP_HASH_L4_SYM_BIT,
+    OVS_OF_GROUP_SUPPORT_BIT,
+    OVS_SAMPLE_REG_SUPPORT_BIT,
 };
 
 enum ovs_feature_value {
@@ -47,16 +48,16 @@ enum ovs_feature_value {
     OVS_DP_METER_SUPPORT = (1 << OVS_DP_METER_SUPPORT_BIT),
     OVS_CT_TUPLE_FLUSH_SUPPORT = (1 << OVS_CT_TUPLE_FLUSH_BIT),
     OVS_DP_HASH_L4_SYM_SUPPORT = (1 << OVS_DP_HASH_L4_SYM_BIT),
+    OVS_OF_GROUP_SUPPORT = (1 << OVS_OF_GROUP_SUPPORT_BIT),
+    OVS_SAMPLE_REG_SUPPORT = (1 << OVS_SAMPLE_REG_SUPPORT_BIT),
 };
 
 void ovs_feature_support_destroy(void);
 bool ovs_feature_is_supported(enum ovs_feature_value feature);
 bool ovs_feature_support_run(const struct smap *ovs_capabilities,
-                             const char *br_name,
-                             const char *db_target);
+                             const char *conn_target, int probe_interval);
 bool ovs_feature_set_discovered(void);
 uint32_t ovs_feature_max_meters_get(void);
 uint32_t ovs_feature_max_select_groups_get(void);
-size_t ovs_features_max_flow_table_prefixes_get(void);
 
 #endif
