@@ -60,17 +60,6 @@ enum ovn_controller_event {
 #define MFF_LOG_LB_AFF_MATCH_LR_IP6_ADDR    MFF_XXREG1
 #define MFF_LOG_LB_AFF_MATCH_PORT           MFF_REG8
 
-#define MFF_LOG_CT_ORIG_NW_DST_ADDR         MFF_REG1   /* REG_ORIG_DIP_IPV4 */
-#define MFF_LOG_CT_ORIG_IP6_DST_ADDR        MFF_XXREG1 /* REG_ORIG_DIP_IPV6 */
-#define MFF_LOG_CT_ORIG_TP_DST_PORT         MFF_REG2   /* REG_ORIG_TP_DPORT
-                                                        * (bits 0..15). */
-#define MFF_LOG_CT_SAVED_STATE              MFF_REG4   /* REG_CT_STATE
-                                                        * (bits 0..8). */
-
-/* Maximum number of networks supported by 4-bit flags.network_id. */
-#define OVN_MAX_NETWORK_ID \
-    ((1 << (MLF_NETWORK_ID_END_BIT - MLF_NETWORK_ID_START_BIT + 1)) - 1)
-
 void ovn_init_symtab(struct shash *symtab);
 
 /* MFF_LOG_FLAGS_REG bit assignments */
@@ -93,10 +82,6 @@ enum mff_log_flags_bits {
     MLF_LOCALNET_BIT = 15,
     MLF_RX_FROM_TUNNEL_BIT = 16,
     MLF_ICMP_SNAT_BIT = 17,
-    MLF_OVERRIDE_LOCAL_ONLY_BIT = 18,
-    MLF_FROM_CTRL_BIT = 19,
-    MLF_NETWORK_ID_START_BIT = 28,
-    MLF_NETWORK_ID_END_BIT = 31,
 };
 
 /* MFF_LOG_FLAGS_REG flag assignments */
@@ -152,12 +137,6 @@ enum mff_log_flags {
     MLF_RX_FROM_TUNNEL = (1 << MLF_RX_FROM_TUNNEL_BIT),
 
     MLF_ICMP_SNAT = (1 << MLF_ICMP_SNAT_BIT),
-
-    MLF_OVERRIDE_LOCAL_ONLY = (1 << MLF_OVERRIDE_LOCAL_ONLY_BIT),
-
-    /* Assign network ID to packet to choose correct network for snat when
-     * lb_force_snat_ip=router_ip. */
-    MLF_NETWORK_ID = (OVN_MAX_NETWORK_ID << MLF_NETWORK_ID_START_BIT),
 };
 
 /* OVN logical fields
@@ -218,6 +197,8 @@ const struct ovn_field *ovn_field_from_name(const char *name);
 #define OVN_CT_NATTED_BIT  1
 #define OVN_CT_LB_SKIP_SNAT_BIT 2
 #define OVN_CT_LB_FORCE_SNAT_BIT 3
+#define OVN_CT_OBS_STAGE_1ST_BIT 4
+#define OVN_CT_OBS_STAGE_END_BIT 5
 
 #define OVN_CT_BLOCKED 1
 #define OVN_CT_NATTED  2
