@@ -194,7 +194,7 @@ enum engine_node_state {
     EN_UNCHANGED, /* Data in the node is valid and didn't change during the
                    * last run.
                    */
-    EN_ABORTED,   /* During the last run, processing was aborted for
+    EN_CANCELED,  /* During the last run, processing was canceled for
                    * this node.
                    */
     EN_STATE_MAX,
@@ -203,7 +203,7 @@ enum engine_node_state {
 struct engine_stats {
     uint64_t recompute;
     uint64_t compute;
-    uint64_t abort;
+    uint64_t cancel;
 };
 
 struct engine_node {
@@ -271,7 +271,7 @@ void engine_init_run(void);
 /* Execute the processing, which should be called in the main loop.
  * Updates the engine node's states accordingly. If 'recompute_allowed' is
  * false and a recompute is required by the current engine run then the engine
- * aborts.
+ * cancels its processing.
  */
 void engine_run(bool recompute_allowed);
 
@@ -297,22 +297,11 @@ void *engine_get_input_data(const char *input_name, struct engine_node *);
 void engine_add_input(struct engine_node *node, struct engine_node *input,
                       bool (*change_handler)(struct engine_node *, void *));
 
-/* Force the engine to recompute everything. It is used
+/* Force the engine to recompute everything if set to true. It is used
  * in circumstances when we are not sure there is change or not, or
  * when there is change but the engine couldn't be executed in that
- * iteration, and the change can't be tracked across iterations. */
-void engine_set_force_recompute(void);
-
-/* Same as "engine_set_force_recompute()", but the poll_loop is woken up
- * immediately and the next engine run is not delayed. */
-void engine_set_force_recompute_immediate(void);
-
-/* Clear the force flag for the next run so the engine does the
- * usual processing without forced full recompute. */
-void engine_clear_force_recompute(void);
-
-/* Returns whether next engine_run() is forced to rempute. */
-bool engine_get_force_recompute(void);
+ * iteration, and the change can't be tracked across iterations */
+void engine_set_force_recompute(bool val);
 
 /* Return the current engine_context. The values in the context can be NULL
  * if the engine is run with allow_recompute == false in the current
@@ -338,8 +327,8 @@ bool engine_has_run(void);
  * has changed; false if nothing has changed. */
 bool engine_has_updated(void);
 
-/* Returns true if during the last engine run we had to abort processing. */
-bool engine_aborted(void);
+/* Returns true if during the last engine run we had to cancel processing. */
+bool engine_canceled(void);
 
 /* Return a pointer to node data accessible for users outside the processing
  * engine. If the node data is not valid (e.g., last engine_run() failed or
@@ -353,7 +342,7 @@ bool engine_aborted(void);
  */
 void *engine_get_data(struct engine_node *node);
 
-/* Return a pointer to node data *without* performing any sanity checks on
+/* Return a pointer to node data *without* performing any coherance checks on
  * the state of the node. This may be used only in specific cases when data
  * is guaranteed to be valid, e.g., immediately after initialization and
  * before the first engine_run().
