@@ -139,12 +139,6 @@ ovn_init_symtab(struct shash *symtab)
                              flags_str);
     snprintf(flags_str, sizeof flags_str, "flags[%d]", MLF_RX_FROM_TUNNEL_BIT);
     expr_symtab_add_subfield(symtab, "flags.tunnel_rx", NULL, flags_str);
-    snprintf(flags_str, sizeof flags_str, "flags[%d..%d]",
-             MLF_NETWORK_ID_START_BIT, MLF_NETWORK_ID_END_BIT);
-    expr_symtab_add_subfield(symtab, "flags.network_id", NULL, flags_str);
-
-    snprintf(flags_str, sizeof flags_str, "flags[%d]", MLF_FROM_CTRL_BIT);
-    expr_symtab_add_subfield(symtab, "flags.from_ctrl", NULL, flags_str);
 
     /* Connection tracking state. */
     expr_symtab_add_field_scoped(symtab, "ct_mark", MFF_CT_MARK, NULL, false,
@@ -171,38 +165,28 @@ ovn_init_symtab(struct shash *symtab)
                                     OVN_CT_STR(OVN_CT_LB_FORCE_SNAT_BIT)
                                     "]",
                                     WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_mark.obs_stage", NULL,
+                                    "ct_mark["
+                                    OVN_CT_STR(OVN_CT_OBS_STAGE_1ST_BIT) ".."
+                                    OVN_CT_STR(OVN_CT_OBS_STAGE_END_BIT)
+                                    "]",
+                                    WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_mark.obs_collector_id", NULL,
+                                    "ct_mark[16..23]", WR_CT_COMMIT);
 
     expr_symtab_add_field_scoped(symtab, "ct_label", MFF_CT_LABEL, NULL,
                                  false, WR_CT_COMMIT);
-    expr_symtab_add_subfield_scoped(symtab, "ct_label.blocked", NULL,
-                                    "ct_label["
-                                        OVN_CT_STR(OVN_CT_BLOCKED_BIT)
-                                    "]",
-                                    WR_CT_COMMIT);
-    expr_symtab_add_subfield_scoped(symtab, "ct_label.natted", NULL,
-                                    "ct_label["
-                                        OVN_CT_STR(OVN_CT_NATTED_BIT)
-                                    "]",
-                                    WR_CT_COMMIT);
     expr_symtab_add_subfield_scoped(symtab, "ct_label.ecmp_reply_eth", NULL,
                                     "ct_label["
                                     OVN_CT_STR(OVN_CT_ECMP_ETH_1ST_BIT) ".."
                                     OVN_CT_STR(OVN_CT_ECMP_ETH_END_BIT) "]",
                                     WR_CT_COMMIT);
-    expr_symtab_add_subfield_scoped(symtab, "ct_label.ecmp_reply_port", NULL,
-                                    "ct_label[80..95]", WR_CT_COMMIT);
     expr_symtab_add_subfield_scoped(symtab, "ct_label.label", NULL,
                                     "ct_label[96..127]", WR_CT_COMMIT);
-    expr_symtab_add_subfield_scoped(symtab, "ct_label.skip_snat", NULL,
-                                    "ct_label["
-                                    OVN_CT_STR(OVN_CT_LB_SKIP_SNAT_BIT)
-                                    "]",
-                                    WR_CT_COMMIT);
-    expr_symtab_add_subfield_scoped(symtab, "ct_label.force_snat", NULL,
-                                    "ct_label["
-                                    OVN_CT_STR(OVN_CT_LB_FORCE_SNAT_BIT)
-                                    "]",
-                                    WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_label.obs_point_id", NULL,
+                                    "ct_label[96..127]", WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_label.obs_unused", NULL,
+                                    "ct_label[0..95]", WR_CT_COMMIT);
 
     expr_symtab_add_field(symtab, "ct_state", MFF_CT_STATE, NULL, false);
 
