@@ -22,6 +22,7 @@
 #include "daemon.h"
 #include "include/ovn/actions.h"
 #include "openvswitch/ofp-parse.h"
+#include "openvswitch/rconn.h"
 #include "openvswitch/vlog.h"
 #include "lib/vswitch-idl.h"
 #include "ovn-dirs.h"
@@ -32,7 +33,6 @@
 #include "stream.h"
 #include "svec.h"
 #include "unixctl.h"
-#include "dummy.h"
 
 VLOG_DEFINE_THIS_MODULE(ovn_util);
 
@@ -865,12 +865,8 @@ ip_address_and_port_from_lb_key(const char *key, char **ip_address,
  * modified or a stage is added to a logical pipeline.
  *
  * This value is also used to handle some backward compatibility during
- * upgrading. It should never decrease or rewind.
- *
- * NOTE: If OVN_NORTHD_PIPELINE_CSUM is updated make sure to double check
- * whether an update of OVN_INTERNAL_MINOR_VER is required. */
-#define OVN_NORTHD_PIPELINE_CSUM "962040559 9628"
-#define OVN_INTERNAL_MINOR_VER 8
+ * upgrading. It should never decrease or rewind. */
+#define OVN_INTERNAL_MINOR_VER 6
 
 /* Returns the OVN version. The caller must free the returned value. */
 char *
@@ -1311,17 +1307,27 @@ ovn_exit_args_finish(struct ovn_exit_args *exit_args)
     free(exit_args->conns);
 }
 
-static void
-ovn_enable_timewarp(struct unixctl_conn *conn, int argc OVS_UNUSED,
-                    const char *argv[] OVS_UNUSED, void *arg OVS_UNUSED)
+bool
+ovn_update_swconn_at(struct rconn *swconn, const char *target,
+                     int probe_interval, const char *where)
 {
-    timeval_dummy_register();
-    unixctl_command_reply(conn, NULL);
-}
+    if (!target) {
+        rconn_disconnect(swconn);
+        return true;
+    }
 
-void
-ovn_debug_commands_register(void)
-{
-    unixctl_command_register("debug/enable-timewarp", "", 0, 0,
-                             ovn_enable_timewarp, NULL);
+    bool notify = false;
+
+    if (strcmp(target, rconn_get_target(swconn))) {
+        VLOG_INFO("%s: connecting to switch: \"%s\"", where, target);
+        rconn_connect(swconn, target, target);
+        notify = true;
+    }
+
+    if (probe_interval != rconn_get_probe_interval(swconn)) {
+        rconn_set_probe_interval(swconn, probe_interval);
+        notify = true;
+    }
+
+    return notify;
 }
