@@ -24,6 +24,7 @@
 /* OVN includes */
 #include "debug.h"
 #include "en-global-config.h"
+#include "en-sampling-app.h"
 #include "include/ovn/features.h"
 #include "ipam.h"
 #include "lib/ovn-nb-idl.h"
@@ -42,8 +43,10 @@ static bool chassis_features_changed(const struct chassis_features *,
 static bool config_out_of_sync(const struct smap *config,
                                const struct smap *saved_config,
                                const char *key, bool must_be_present);
-static bool check_nb_options_out_of_sync(const struct nbrec_nb_global *,
-                                         struct ed_type_global_config *);
+static bool check_nb_options_out_of_sync(
+    const struct nbrec_nb_global *,
+    struct ed_type_global_config *,
+    const struct sampling_app_table *);
 static void update_sb_config_options_to_sbrec(struct ed_type_global_config *,
                                               const struct sbrec_sb_global *);
 
@@ -72,6 +75,9 @@ en_global_config_run(struct engine_node *node , void *data)
         EN_OVSDB_GET(engine_get_input("SB_sb_global", node));
     const struct sbrec_chassis_table *sbrec_chassis_table =
         EN_OVSDB_GET(engine_get_input("SB_chassis", node));
+    const struct ed_type_sampling_app_data *sampling_app_data =
+        engine_get_input_data("sampling_app", node);
+    const struct sampling_app_table *sampling_apps = &sampling_app_data->apps;
 
     struct ed_type_global_config *config_data = data;
 
@@ -116,7 +122,9 @@ en_global_config_run(struct engine_node *node , void *data)
     }
 
     char *max_tunid = xasprintf("%d",
-        get_ovn_max_dp_key_local(sbrec_chassis_table));
+                                get_ovn_max_dp_key_local(
+                                    is_vxlan_mode(&nb->options,
+                                                  sbrec_chassis_table)));
     smap_replace(options, "max_tunid", max_tunid);
     free(max_tunid);
 
@@ -143,7 +151,8 @@ en_global_config_run(struct engine_node *node , void *data)
         build_chassis_features(sbrec_chassis_table, &config_data->features);
     }
 
-    init_debug_config(nb);
+    init_debug_config(nb, sampling_app_get_id(sampling_apps,
+                                              SAMPLING_APP_DROP_DEBUG));
 
     const struct sbrec_sb_global *sb =
         sbrec_sb_global_table_first(sb_global_table);
@@ -184,6 +193,9 @@ global_config_nb_global_handler(struct engine_node *node, void *data)
         EN_OVSDB_GET(engine_get_input("NB_nb_global", node));
     const struct sbrec_sb_global_table *sb_global_table =
         EN_OVSDB_GET(engine_get_input("SB_sb_global", node));
+    const struct ed_type_sampling_app_data *sampling_app_data =
+        engine_get_input_data("sampling_app", node);
+    const struct sampling_app_table *sampling_apps = &sampling_app_data->apps;
 
     const struct nbrec_nb_global *nb =
         nbrec_nb_global_table_first(nb_global_table);
@@ -246,7 +258,7 @@ global_config_nb_global_handler(struct engine_node *node, void *data)
         return false;
     }
 
-    if (check_nb_options_out_of_sync(nb, config_data)) {
+    if (check_nb_options_out_of_sync(nb, config_data, sampling_apps)) {
         config_data->tracked_data.nb_options_changed = true;
     }
 
@@ -364,15 +376,12 @@ static void
 northd_enable_all_features(struct ed_type_global_config *data)
 {
     data->features = (struct chassis_features) {
-        .ct_no_masked_label = true,
         .mac_binding_timestamp = true,
-        .ct_lb_related = true,
         .fdb_timestamp = true,
         .ls_dpg_column = true,
         .ct_commit_nat_v2 = true,
         .ct_commit_to_zone = true,
-        .ct_next_zone = true,
-        .ct_state_save = true,
+        .sample_with_reg = true,
     };
 }
 
@@ -390,14 +399,6 @@ build_chassis_features(const struct sbrec_chassis_table *sbrec_chassis_table,
             continue;
         }
 
-        bool ct_no_masked_label =
-            smap_get_bool(&chassis->other_config,
-                          OVN_FEATURE_CT_NO_MASKED_LABEL,
-                          false);
-        if (!ct_no_masked_label && chassis_features->ct_no_masked_label) {
-            chassis_features->ct_no_masked_label = false;
-        }
-
         bool mac_binding_timestamp =
             smap_get_bool(&chassis->other_config,
                           OVN_FEATURE_MAC_BINDING_TIMESTAMP,
@@ -407,15 +408,6 @@ build_chassis_features(const struct sbrec_chassis_table *sbrec_chassis_table,
             chassis_features->mac_binding_timestamp = false;
         }
 
-        bool ct_lb_related =
-            smap_get_bool(&chassis->other_config,
-                          OVN_FEATURE_CT_LB_RELATED,
-                          false);
-        if (!ct_lb_related &&
-            chassis_features->ct_lb_related) {
-            chassis_features->ct_lb_related = false;
-        }
-
         bool fdb_timestamp =
             smap_get_bool(&chassis->other_config,
                           OVN_FEATURE_FDB_TIMESTAMP,
@@ -452,22 +444,13 @@ build_chassis_features(const struct sbrec_chassis_table *sbrec_chassis_table,
             chassis_features->ct_commit_to_zone = false;
         }
 
-        bool ct_next_zone =
+        bool sample_with_reg =
                 smap_get_bool(&chassis->other_config,
-                              OVN_FEATURE_CT_NEXT_ZONE,
+                              OVN_FEATURE_SAMPLE_WITH_REGISTERS,
                               false);
-        if (!ct_next_zone &&
-            chassis_features->ct_next_zone) {
-            chassis_features->ct_next_zone = false;
-        }
-
-        bool ct_state_save =
-                smap_get_bool(&chassis->other_config,
-                              OVN_FEATURE_CT_STATE_SAVE,
-                              false);
-        if (!ct_state_save &&
-            chassis_features->ct_state_save) {
-            chassis_features->ct_state_save = false;
+        if (!sample_with_reg &&
+            chassis_features->sample_with_reg) {
+            chassis_features->sample_with_reg = false;
         }
     }
 }
@@ -498,8 +481,10 @@ config_out_of_sync(const struct smap *config, const struct smap *saved_config,
 }
 
 static bool
-check_nb_options_out_of_sync(const struct nbrec_nb_global *nb,
-                             struct ed_type_global_config *config_data)
+check_nb_options_out_of_sync(
+    const struct nbrec_nb_global *nb,
+    struct ed_type_global_config *config_data,
+    const struct sampling_app_table *sampling_apps)
 {
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "mac_binding_removal_limit", false)) {
@@ -533,13 +518,16 @@ check_nb_options_out_of_sync(const struct nbrec_nb_global *nb,
 
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "debug_drop_domain_id", false)) {
-        init_debug_config(nb);
+        init_debug_config(nb, sampling_app_get_id(sampling_apps,
+                                                  SAMPLING_APP_DROP_DEBUG));
+
         return true;
     }
 
     if (config_out_of_sync(&nb->options, &config_data->nb_options,
                            "debug_drop_collector_set", false)) {
-        init_debug_config(nb);
+        init_debug_config(nb, sampling_app_get_id(sampling_apps,
+                                                  SAMPLING_APP_DROP_DEBUG));
         return true;
     }
 
@@ -553,6 +541,16 @@ check_nb_options_out_of_sync(const struct nbrec_nb_global *nb,
         return true;
     }
 
+    if (config_out_of_sync(&nb->options, &config_data->nb_options,
+                           "vxlan_mode", false)) {
+        return true;
+    }
+
+    if (config_out_of_sync(&nb->options, &config_data->nb_options,
+                           "always_tunnel", false)) {
+        return true;
+    }
+
     return false;
 }
 
@@ -565,16 +563,6 @@ update_sb_config_options_to_sbrec(struct ed_type_global_config *config_data,
     smap_destroy(options);
     smap_clone(options, &config_data->nb_options);
 
-    /* Inform ovn-controllers whether LB flows will use ct_mark (i.e., only
-     * if all chassis support it).  If not explicitly present in the database
-     * the default value to be used for this option is 'true'.
-     */
-    if (!config_data->features.ct_no_masked_label) {
-        smap_replace(options, "lb_hairpin_use_ct_mark", "false");
-    } else {
-        smap_remove(options, "lb_hairpin_use_ct_mark");
-    }
-
     /* Hackaround SB_global.options overwrite by NB_Global.options for
      * 'sbctl_probe_interval' option.
      */
@@ -596,18 +584,10 @@ static bool
 chassis_features_changed(const struct chassis_features *present,
                          const struct chassis_features *updated)
 {
-    if (present->ct_no_masked_label != updated->ct_no_masked_label) {
-        return true;
-    }
-
     if (present->mac_binding_timestamp != updated->mac_binding_timestamp) {
         return true;
     }
 
-    if (present->ct_lb_related != updated->ct_lb_related) {
-        return true;
-    }
-
     if (present->fdb_timestamp != updated->fdb_timestamp) {
         return true;
     }
