@@ -120,7 +120,7 @@ static const char *rbac_svc_monitor_auth_update[] =
 static const char *rbac_igmp_group_auth[] =
     {"chassis_name"};
 static const char *rbac_igmp_group_update[] =
-    {"address", "chassis", "datapath", "ports"};
+    {"address", "protocol", "chassis", "datapath", "ports"};
 static const char *rbac_bfd_auth[] =
     {"chassis_name"};
 static const char *rbac_bfd_update[] =
@@ -277,16 +277,20 @@ static struct gen_opts_map supported_dhcpv6_opts[] = {
     DHCPV6_OPT_FQDN,
 };
 
-/*
- * Compare predefined permission against RBAC_Permission record.
- * Returns true if match, false otherwise.
- */
 static bool
-ovn_rbac_match_perm(const struct sbrec_rbac_permission *perm,
-                    const struct rbac_perm_cfg *pcfg)
+ovn_rbac_validate_perm(const struct sbrec_rbac_permission *perm)
 {
+    struct rbac_perm_cfg *pcfg;
     int i, j, n_found;
 
+    for (pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
+        if (!strcmp(perm->table, pcfg->table)) {
+            break;
+        }
+    }
+    if (!pcfg->table) {
+        return false;
+    }
     if (perm->n_authorization != pcfg->n_auth ||
         perm->n_update != pcfg->n_update) {
         return false;
@@ -322,71 +326,54 @@ ovn_rbac_match_perm(const struct sbrec_rbac_permission *perm,
         return false;
     }
 
+    /* Success, db state matches expected state */
+    pcfg->row = perm;
     return true;
 }
 
-/*
- * Search predefined permission pcfg in the RBAC_Permission.
- * If there is no record that match, recover the permission.
- */
 static void
-ovn_rbac_validate_perm(struct rbac_perm_cfg *pcfg,
-                       struct ovsdb_idl_txn *ovnsb_txn,
-                       struct ovsdb_idl *ovnsb_idl)
+ovn_rbac_create_perm(struct rbac_perm_cfg *pcfg,
+                     struct ovsdb_idl_txn *ovnsb_txn,
+                     const struct sbrec_rbac_role *rbac_role)
 {
-    const struct sbrec_rbac_permission *perm_row;
+    struct sbrec_rbac_permission *rbac_perm;
 
-    SBREC_RBAC_PERMISSION_FOR_EACH (perm_row, ovnsb_idl) {
-        if (!strcmp(perm_row->table, pcfg->table)
-            && ovn_rbac_match_perm(perm_row, pcfg)) {
-            pcfg->row = perm_row;
-
-            return;
-        }
-    }
-
-    pcfg->row = sbrec_rbac_permission_insert(ovnsb_txn);
-    sbrec_rbac_permission_set_table(pcfg->row, pcfg->table);
-    sbrec_rbac_permission_set_authorization(pcfg->row,
+    rbac_perm = sbrec_rbac_permission_insert(ovnsb_txn);
+    sbrec_rbac_permission_set_table(rbac_perm, pcfg->table);
+    sbrec_rbac_permission_set_authorization(rbac_perm,
                                             pcfg->auth,
                                             pcfg->n_auth);
-    sbrec_rbac_permission_set_insert_delete(pcfg->row, pcfg->insdel);
-    sbrec_rbac_permission_set_update(pcfg->row,
+    sbrec_rbac_permission_set_insert_delete(rbac_perm, pcfg->insdel);
+    sbrec_rbac_permission_set_update(rbac_perm,
                                      pcfg->update,
                                      pcfg->n_update);
+    sbrec_rbac_role_update_permissions_setkey(rbac_role, pcfg->table,
+                                              rbac_perm);
 }
 
-/*
- * Make sure that DB Role 'ovn-controller' exists, has no duplicates
- * permission list exactly match to predefined permissions.  Recreate if
- * matching fails.
- */
 static void
 check_and_update_rbac(struct ovsdb_idl_txn *ovnsb_txn,
                       struct ovsdb_idl *ovnsb_idl)
 {
     const struct sbrec_rbac_role *rbac_role = NULL;
+    const struct sbrec_rbac_permission *perm_row;
     const struct sbrec_rbac_role *role_row;
+    struct rbac_perm_cfg *pcfg;
 
-    /*
-     * Make sure predefined permissions are presented in the RBAC_Permissions
-     * table. Otherwise create consistent permissions.
-     */
-    for (struct rbac_perm_cfg *pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
-        ovn_rbac_validate_perm(pcfg, ovnsb_txn, ovnsb_idl);
+    for (pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
+        pcfg->row = NULL;
     }
 
-    /*
-     * Make sure the role 'ovn-controller' is presented in the RBAC_Role table.
-     * Otherwise create the role. Remove duplicates if any.
-     */
+    SBREC_RBAC_PERMISSION_FOR_EACH_SAFE (perm_row, ovnsb_idl) {
+        if (!ovn_rbac_validate_perm(perm_row)) {
+            sbrec_rbac_permission_delete(perm_row);
+        }
+    }
     SBREC_RBAC_ROLE_FOR_EACH_SAFE (role_row, ovnsb_idl) {
-        if (!strcmp(role_row->name, "ovn-controller")) {
-            if (rbac_role) {
-                sbrec_rbac_role_delete(role_row);
-            } else {
-                rbac_role = role_row;
-            }
+        if (strcmp(role_row->name, "ovn-controller")) {
+            sbrec_rbac_role_delete(role_row);
+        } else {
+            rbac_role = role_row;
         }
     }
 
@@ -395,38 +382,10 @@ check_and_update_rbac(struct ovsdb_idl_txn *ovnsb_txn,
         sbrec_rbac_role_set_name(rbac_role, "ovn-controller");
     }
 
-    /*
-     * Make sure the permission list attached to the role 'ovn-controller'
-     * exactly matches to predefined permissions.
-     * Reassign permission list to the role if any difference has found.
-     */
-    if (ARRAY_SIZE(rbac_perm_cfg) - 1 != rbac_role->n_permissions) {
-        goto rebuild_role_perms;
-    }
-
-    for (struct rbac_perm_cfg *pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
-        size_t i;
-        for (i = 0; i < rbac_role->n_permissions; i++) {
-            if (!strcmp(pcfg->table, rbac_role->key_permissions[i])
-                && pcfg->row == rbac_role->value_permissions[i]) {
-                break;
-            }
+    for (pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
+        if (!pcfg->row) {
+            ovn_rbac_create_perm(pcfg, ovnsb_txn, rbac_role);
         }
-
-        if (i == rbac_role->n_permissions) {
-            goto rebuild_role_perms;
-        }
-    }
-
-    return;
-
-rebuild_role_perms:
-    sbrec_rbac_role_set_permissions(rbac_role, NULL, NULL, 0);
-
-    for (struct rbac_perm_cfg *pcfg = rbac_perm_cfg; pcfg->table; pcfg++) {
-        sbrec_rbac_role_update_permissions_setkey(rbac_role,
-                                                  pcfg->table,
-                                                  pcfg->row);
     }
 }
 
@@ -845,7 +804,6 @@ main(int argc, char *argv[])
     unixctl_command_register("parallel-build/get-n-threads", "", 0, 0,
                              ovn_northd_get_thread_count_cmd,
                              NULL);
-    ovn_debug_commands_register();
 
     daemonize_complete();
 
@@ -974,7 +932,7 @@ main(int argc, char *argv[])
             if (new_ovnnb_cond_seqno != ovnnb_cond_seqno) {
                 if (!new_ovnnb_cond_seqno) {
                     VLOG_INFO("OVN NB IDL reconnected, force recompute.");
-                    inc_proc_northd_force_recompute();
+                    eng_ctx.recompute = true;
                 }
                 ovnnb_cond_seqno = new_ovnnb_cond_seqno;
             }
@@ -987,7 +945,7 @@ main(int argc, char *argv[])
             if (new_ovnsb_cond_seqno != ovnsb_cond_seqno) {
                 if (!new_ovnsb_cond_seqno) {
                     VLOG_INFO("OVN SB IDL reconnected, force recompute.");
-                    inc_proc_northd_force_recompute();
+                    eng_ctx.recompute = true;
                 }
                 ovnsb_cond_seqno = new_ovnsb_cond_seqno;
             }
@@ -1011,6 +969,7 @@ main(int argc, char *argv[])
                     int64_t loop_start_time = time_wall_msec();
                     activity = inc_proc_northd_run(ovnnb_txn, ovnsb_txn,
                                                    &eng_ctx);
+                    eng_ctx.recompute = false;
                     check_and_add_supported_dhcp_opts_to_sb_db(
                                  ovnsb_txn, ovnsb_idl_loop.idl);
                     check_and_add_supported_dhcpv6_opts_to_sb_db(
@@ -1023,7 +982,7 @@ main(int argc, char *argv[])
                                             ovnsb_idl_loop.idl,
                                             ovnnb_txn, ovnsb_txn,
                                             &ovnsb_idl_loop);
-                } else if (!inc_proc_northd_get_force_recompute()) {
+                } else if (!eng_ctx.recompute) {
                     clear_idl_track = false;
                 }
 
@@ -1032,13 +991,13 @@ main(int argc, char *argv[])
                 if (!ovsdb_idl_loop_commit_and_wait(&ovnnb_idl_loop)) {
                     VLOG_INFO("OVNNB commit failed, "
                               "force recompute next time.");
-                    inc_proc_northd_force_recompute_immediate();
+                    eng_ctx.recompute = true;
                 }
 
                 if (!ovsdb_idl_loop_commit_and_wait(&ovnsb_idl_loop)) {
                     VLOG_INFO("OVNSB commit failed, "
                               "force recompute next time.");
-                    inc_proc_northd_force_recompute_immediate();
+                    eng_ctx.recompute = true;
                 }
                 run_memory_trimmer(ovnnb_idl_loop.idl, activity);
             } else {
@@ -1047,7 +1006,7 @@ main(int argc, char *argv[])
                 ovsdb_idl_loop_commit_and_wait(&ovnsb_idl_loop);
 
                 /* Force a full recompute next time we become active. */
-                inc_proc_northd_force_recompute();
+                eng_ctx.recompute = true;
             }
         } else {
             /* ovn-northd is paused
@@ -1071,7 +1030,7 @@ main(int argc, char *argv[])
             ovsdb_idl_wait(ovnsb_idl_loop.idl);
 
             /* Force a full recompute next time we become active. */
-            inc_proc_northd_force_recompute_immediate();
+            eng_ctx.recompute = true;
         }
 
         if (clear_idl_track) {
